package de.uka.ipd.sdq.cip.configuration;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import de.uka.ipd.sdq.cip.ConstantsContainer;
import de.uka.ipd.sdq.workflow.IJob;
import de.uka.ipd.sdq.workflow.mdsd.blackboard.ResourceSetPartition;

public class CompletionConfiguration {
	protected List<Transformation> transformations;
	protected String inputPartitionName;
	protected String tracesPartitionName;
	protected ResourceSetPartition lastPartition;
	protected String projectID;
	protected boolean isDebug;
	protected boolean deleteProject;
	protected Collection<IJob> afterJobs;
	protected Collection<Object> metaModels;
	
	@SuppressWarnings("unchecked")
	public CompletionConfiguration(Map<String, Object> configuration) {
		try {
			projectID = (String) configuration.get(
					ConstantsContainer.COMPLETION_PROJECT_ID);
			isDebug = (Boolean) configuration.get(ConstantsContainer.COMPLETION_QVT_VERBOSE_LOGGING);
			deleteProject = (Boolean) configuration.get(ConstantsContainer.COMPLETION_DELETE_PLUGIN);
			Collection<String> transformationStrings = (Collection<String>) configuration.get(ConstantsContainer.COMPLETION_CONFIG);
			transformations = new ArrayList<Transformation>();
			for(String configString:transformationStrings)
			{
				Transformation transformation = Transformation.fromDataString(configString);
				if(transformation.enabled)
					transformations.add(transformation);
			}
		} catch (Exception e) {
			throw new RuntimeException("Setting up properties failed, please check launch config (check all tabs).", e);
		}
	}

	public List<Transformation> getTransformations() {
		return transformations;
	}

	public String getTracesPartitionName() {
		return tracesPartitionName;
	}
	
	public String getInputPartitionName(){
		return inputPartitionName;
	}
	
	public void setInputPartitionName(
			String inputPartitionName) {
		this.inputPartitionName = inputPartitionName;
		
	}
	
	public Collection<IJob> getAfterJobs() {
		return afterJobs;
	}

	public void setAfterJobs(Collection<IJob> afterJobs) {
		this.afterJobs = afterJobs;
	}
	
	public void setAfterJobs(IJob[] afterJobs) {
		this.afterJobs = Arrays.asList(afterJobs);
	}

	public ResourceSetPartition getLastPartition() {
		return lastPartition;
	}

	public void setLastPartition(
			ResourceSetPartition lastPartition) {
		this.lastPartition = lastPartition;
		
	}
	
	public Collection<Object> getMetaModels() {
		return metaModels;
	}
	
	public void setMetaModels(Collection<Object> metaModels) {
		this.metaModels = metaModels;		
	}
	
	public boolean getDebug() {
		return isDebug;
	}
	
	public String getProjectID() {
		return projectID;
	}

	public boolean shouldDeleteGeneratedProjectOnCleanup() {
		return deleteProject;
	}
}
