package de.uka.ipd.sdq.cip.runtime.dialogs;

import java.io.File;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.emf.common.util.URI;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.FilteredResourcesSelectionDialog;

import de.uka.ipd.sdq.cip.completions.RegisteredCompletion;
import de.uka.ipd.sdq.cip.runtime.ConstantsContainer;
import de.uka.ipd.sdq.cip.runtime.dialogs.CompletionConfigNode.UserDefined;
import de.uka.ipd.sdq.cip.runtime.dialogs.CompletionConfigNode.UserDefinedID;

public class CompletionConfigurationPage extends WizardPage {

	private FileSelectionWidget file1;
	private FileSelectionWidget file2;
	private FileSelectionWidget file3;

	protected CompletionConfigurationPage(String pageName) {
		super(pageName);
	}

	@Override
	public void createControl(Composite parent) {
	Composite mainComponent = new Composite(parent, SWT.NONE);
		
		GridLayout mainlayout = new GridLayout();
		mainComponent.setLayout(mainlayout);
		
		file1 = new FileSelectionWidget(mainComponent, SWT.NONE);
		GridData gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = false;
		gd.verticalAlignment = SWT.FILL;
		gd.horizontalAlignment = SWT.FILL;
		file1.setLayoutData(gd);
		
		file2 = new FileSelectionWidget(mainComponent, SWT.NONE);
		gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = false;
		gd.verticalAlignment = SWT.FILL;
		gd.horizontalAlignment = SWT.FILL;
		file2.setLayoutData(gd);
	
		file3 = new FileSelectionWidget(mainComponent, SWT.NONE);
		gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = false;
		gd.verticalAlignment = SWT.FILL;
		gd.horizontalAlignment = SWT.FILL;
		file3.setLayoutData(gd);
		
		setControl(mainComponent);

	}
	
	public boolean canFinish() {
		CompletionSelectionPage prevpage = (CompletionSelectionPage) getWizard().getPreviousPage(this);
		Object element = prevpage.getSelection();
		if(element instanceof RegisteredCompletion) {
			if(file1.getText() != null)
				return true;
		}
		if(element instanceof UserDefined) {
			UserDefined userDefined = (UserDefined) element; 
			if(userDefined.getID() == UserDefinedID.ANNOTATED) {
				if(file1.getText() != null && file2.getText() != null && file3.getText() != null)
					return true;
			}
			else {
				if(file1.getText() != null)
					return true;
			}
		}
		return false;
	}
	
	@Override
	public void setVisible(boolean visible) {
		CompletionSelectionPage prevpage = (CompletionSelectionPage) getWizard().getPreviousPage(this);
		Object element = prevpage.getSelection();
		
		file2.setVisible(false);
		file3.setVisible(false);
		
		if(element instanceof RegisteredCompletion) {
			file1.setCaption(ConstantsContainer.MARK_MODEL_CAPTION);
			file1.setFileExtensions(null);
		}
		else if(element instanceof UserDefined) {
			UserDefined userDefined = (UserDefined) element; 
			if(userDefined.getID() == UserDefinedID.ANNOTATED) {
				file1.setCaption(ConstantsContainer.COMPLETION_FILE_CAPTION);
				file1.setFileExtensions(ConstantsContainer.COMPLETION_FILE_EXTENSIONS);
				file2.setVisible(true);
				file2.setCaption(ConstantsContainer.MARK_MODEL_CAPTION);
				file2.setFileExtensions(null);
				file3.setVisible(true);
				file3.setCaption(ConstantsContainer.MARK_METAMODEL_CAPTION);
				file3.setFileExtensions(null);
			}
			else if(userDefined.getID() == UserDefinedID.PLAIN) {
				file1.setCaption(ConstantsContainer.COMPLETION_FILE_CAPTION);
				file1.setFileExtensions(ConstantsContainer.COMPLETION_FILE_EXTENSIONS);
			}
			else if(userDefined.getID() == UserDefinedID.FEATURECONFIG) {
				file1.setCaption(ConstantsContainer.FEATURECONFIG_FILE_CAPTION);
				file1.setFileExtensions(ConstantsContainer.FEATURECONFIG_FILE_EXTENSIONS);
			}
		}
		super.setVisible(visible);
	}
	
	public String getFile1Text() {
		return file1.getText();
	}
	
	public String getFile2Text() {
		return file2.getText();
	}

	public String getFile3Text() {
		return file3.getText();
	}
	
	class FileSelectionWidget extends Composite{

		private Label label;
		private Text textfield;
		private Button workspaceButton;
		private Button filesystemButton;
		
		private String[] fileExtensions;

		public FileSelectionWidget(Composite parent, int style) {
			super(parent, style);
			GridLayout layout = new GridLayout(3,false);
			setLayout(layout);
			
			label = new Label(this, SWT.LEFT);
			label.setText("DUMMY");
			GridData gd = new GridData();
			gd.grabExcessHorizontalSpace = true;
			gd.horizontalAlignment = SWT.FILL;
			label.setLayoutData(gd);
			
			workspaceButton = createButton(this, WORKSPACE);
			gd = new GridData();
			workspaceButton.setLayoutData(gd);
			
			filesystemButton = createButton(this, FILESYSTEM);
			gd = new GridData();
			filesystemButton.setLayoutData(gd);
			
			textfield = new Text(this,SWT.BORDER);
			gd = new GridData();
			gd.horizontalSpan = layout.numColumns;
			gd.grabExcessHorizontalSpace = true;
			gd.horizontalAlignment = SWT.FILL;
			gd.verticalAlignment = SWT.TOP;
			textfield.setLayoutData(gd);
			
		}
		
		public void setCaption(String caption) {
			label.setText(caption);
		}
		public void setFileExtensions(String[] fileExtensions) {
			this.fileExtensions = fileExtensions;
		}
		
		public void setText(String text) {
			textfield.setText(text);
		}
		
		public String getText() {
			return textfield.getText();
		}
		
		private final static int FILESYSTEM = 2;
		private final static int WORKSPACE = 1;
		
		private Button createButton(Composite parent, int style) {
			Button button = new Button(parent,SWT.PUSH);
			if((style & WORKSPACE) == WORKSPACE) {
				button.setText("Browse &Workspace...");
			}
			else {
				button.setText("Browse &File System...");
				
			}
			button.addSelectionListener(new ButtonSelectionListener(this,style));
			return button;
		}

		public boolean browseWorkspace() {
			/*ResourceSelectionDialog resourceSelectionDialog = new ResourceSelectionDialog(
			  getShell(),
			  ResourcesPlugin.getWorkspace().getRoot(),
			  "File Selection");*/
			
			FilteredResourcesSelectionDialog resourceSelectionDialog = new FilteredResourcesSelectionDialog(getShell(), false, ResourcesPlugin.getWorkspace().getRoot(),IResource.DEPTH_INFINITE | IResource.FILE);
			resourceSelectionDialog.open();
			Object [] result = resourceSelectionDialog.getResult();
			if (result != null)
			{
			  StringBuffer  text = new StringBuffer ();
			  int length = result.length;
			  for (int i = 0; i < length; ++i)
			  {
			    IResource resource = (IResource)result[i];
			    //if (isValidWorkspaceResource(resource))
			    {
			      text.append(URI.createURI(URI.createPlatformResourceURI(resource.getFullPath().toString(),true).toString(), true));
			      text.append(" ");
			    }
			  }
			  textfield.setText(text.toString().trim());
			  return true;
			}
			return false;
		}

		private boolean browseFileSystem() {
			FileDialog fileDialog = new FileDialog(getShell(), SWT.OPEN | SWT.MULTI );
			fileDialog.setFilterExtensions(fileExtensions);
			
			if (textfield.getText() != null)
			{
			  fileDialog.setFileName(textfield.getText());
			}
			
			if (fileDialog.open() != null && fileDialog.getFileNames().length > 0)
			{
			  String [] fileNames = fileDialog.getFileNames();
			  StringBuffer  text = new StringBuffer ();
			  for (int i = 0; i < fileNames.length; ++i)
			  {
			    String  filePath = fileDialog.getFilterPath() + File.separator + fileNames[i];
			    text.append(URI.createFileURI(filePath).toString());
			    text.append(" ");
			  }
			  textfield.setText(text.toString().trim());
			  
			  return true;
			}
			return false;
		}
		
		private class ButtonSelectionListener implements SelectionListener {
			FileSelectionWidget widget;
			int style;

			public ButtonSelectionListener(FileSelectionWidget fileSelectionWidget, int style) {
				widget = fileSelectionWidget;
				this.style = style;
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				
			}

			@Override
			public void widgetSelected(SelectionEvent e) {
				if((style & WORKSPACE) == WORKSPACE) {
					widget.browseWorkspace();
				}
				else {
					widget.browseFileSystem();
					
				}
			}
			
		}
	}
	
	
	

}
