package de.uka.ipd.sdq.cip.runtime.dialogs;

import org.eclipse.jface.wizard.Wizard;

import de.uka.ipd.sdq.cip.completions.RegisteredCompletion;
import de.uka.ipd.sdq.cip.configuration.Transformation;
import de.uka.ipd.sdq.cip.configuration.Transformation.TransformationType;
import de.uka.ipd.sdq.cip.runtime.ConstantsContainer;
import de.uka.ipd.sdq.cip.runtime.RunConfigImages;
import de.uka.ipd.sdq.cip.runtime.dialogs.CompletionConfigNode.FeatureConfig;
import de.uka.ipd.sdq.cip.runtime.dialogs.CompletionConfigNode.UserDefined;
import de.uka.ipd.sdq.cip.runtime.dialogs.CompletionConfigNode.UserDefinedID;

public class NewCompletionWizard extends Wizard {
	
	private CompletionSelectionPage completionSelectionPage;
	private CompletionConfigurationPage completionConfigurationPage;
	private Transformation transformation = null;

	@Override
	public boolean performFinish() {
		transformation = new Transformation();
		Object element = completionSelectionPage.getSelection();
		if(element instanceof FeatureConfig) {
			FeatureConfig featureConfig = (FeatureConfig) element;
			transformation.enabled = true;
			transformation.type = TransformationType.HOT;
			transformation.configPath = featureConfig.getFileURI();
		}
		else if(element instanceof RegisteredCompletion) {
			RegisteredCompletion completion = (RegisteredCompletion) element;
			transformation.enabled = true;
			transformation.type = TransformationType.ANNOTATED;
			transformation.completionPath = completion.getQvtscript();
			if(completion.containsCategory(ConstantsContainer.MARK_COMPLETION))
				transformation.configPath = completionConfigurationPage.getFile1Text();
				transformation.metamodel = completion.getMetamodel();
		}
		else if(element instanceof UserDefined) {
			UserDefined userDefined = (UserDefined) element; 
			if(userDefined.getID() == UserDefinedID.ANNOTATED) {
				transformation.enabled = true;
				transformation.type = TransformationType.ANNOTATED;
				transformation.completionPath = completionConfigurationPage.getFile1Text();
				transformation.configPath = completionConfigurationPage.getFile2Text();
				transformation.metamodel = completionConfigurationPage.getFile3Text();
			}
			else if(userDefined.getID() == UserDefinedID.PLAIN) {
				transformation.enabled = true;
				transformation.type = TransformationType.PLAIN;
				transformation.completionPath = completionConfigurationPage.getFile1Text();
			}
			else if(userDefined.getID() == UserDefinedID.FEATURECONFIG) {
				transformation.enabled = true;
				transformation.type = TransformationType.HOT;
				transformation.configPath = completionConfigurationPage.getFile1Text();
			}
		}
		return true;
	}
	
	@Override
	public void addPages() {
		setDefaultPageImageDescriptor(RunConfigImages.getChiliIconDescriptor());
		completionSelectionPage = new CompletionSelectionPage("CompletionType");
		completionSelectionPage.setTitle("Select Completion Type");
		completionSelectionPage.setDescription("Select a Completion type and configure it.");
		addPage(completionSelectionPage);
		completionConfigurationPage = new CompletionConfigurationPage("CompletionConfiguration");
		completionConfigurationPage.setTitle("Set Completion Configuration");
		completionConfigurationPage.setDescription("Add configuration files to the completion.");
		addPage(completionConfigurationPage);
	}

	@Override
	public boolean canFinish() {
		if(completionSelectionPage.canFinish())
			return true;
		if(completionConfigurationPage.canFinish())
			return true;
		return false;
	}

	public Transformation getTransformation() {
		return transformation;
	}

}
