/**
 * 
 */
package de.uka.ipd.sdq.cip.runtime.runconfig.tabs;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.emf.common.util.URI;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.DialogCellEditor;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.dialogs.ElementTreeSelectionDialog;
import org.eclipse.ui.model.WorkbenchContentProvider;
import org.eclipse.ui.model.WorkbenchLabelProvider;

import de.uka.ipd.sdq.cip.configuration.Transformation;
import de.uka.ipd.sdq.cip.ConstantsContainer;
import de.uka.ipd.sdq.cip.runtime.RunConfigImages;
import de.uka.ipd.sdq.cip.runtime.dialogs.EditCompletionDialog;
import de.uka.ipd.sdq.cip.runtime.dialogs.NewCompletionWizard;

/**
 * @author thomass
 *
 */
public class CipCompletionTab extends AbstractCipLaunchConfigurationTab {
	
	ArrayList<Transformation> transformations;
	
	private TableViewer completionsViewer;

	private Button newbutton;

	private Button removebutton;

	private Button editbutton;

	private Button upbutton;

	private Button downbutton;

	private ModifyListener modifyListener;

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#createControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createControl(Composite parent) {
		
		modifyListener = new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setTabModified();
			}
		};
		
		Composite mainComponent = new Composite(parent,SWT.NONE);
		setControl(mainComponent);
		
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.marginWidth = 5;
		layout.marginHeight = 5;
		
		mainComponent.setFont(parent.getFont());
		mainComponent.setLayout(layout);
		
		Label label = new Label(mainComponent,SWT.LEFT);
		label.setText("Transformations:");
		
		GridData gd = new GridData();
		gd.horizontalSpan = 2;
		label.setLayoutData(gd);
		
		completionsViewer = createTable(mainComponent,new String[]{"Completion","Annotation"});
		//createColumns(completionsViewer);
		
		newbutton = createPushButton(mainComponent, "N&ew...", null);
		newbutton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				newCompletion();
			}
		});
		editbutton = createPushButton(mainComponent, "E&dit...", null);
		editbutton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				editCompletion();
			}
		});
		removebutton = createPushButton(mainComponent, "Rem&ove", null);
		removebutton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				removeCompletion();
			}
		});
		//createSeparator(mainComponent, 1);
		upbutton = createPushButton(mainComponent, "&Up", null);
		upbutton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				upCompletion();
			}
		});
		downbutton = createPushButton(mainComponent, "&Down", null);
		downbutton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				downCompletion();
			}
		});
		
	}

	protected void setTabModified() {
		this.setDirty(true);
		this.updateLaunchConfigurationDialog();
		
	}

	protected void downCompletion() {
		IStructuredSelection selection = (IStructuredSelection) completionsViewer.getSelection();
		if(!selection.iterator().hasNext())
			return;
		Transformation transformation = (Transformation) selection.iterator().next();
		int pos = transformations.indexOf(transformation);
		if(pos < transformations.size()) {
			transformations.set(pos, transformations.get(pos+1));
			transformations.set(pos+1, transformation);
			completionsViewer.refresh();
		}
		
	}

	protected void upCompletion() {
		IStructuredSelection selection = (IStructuredSelection) completionsViewer.getSelection();
		if(!selection.iterator().hasNext())
			return;
		Transformation transformation = (Transformation) selection.iterator().next();
		int pos = transformations.indexOf(transformation);
		if(pos > 0) {
			transformations.set(pos, transformations.get(pos-1));
			transformations.set(pos-1, transformation);
			completionsViewer.refresh();
		}
	}

	protected void editCompletion() {
		IStructuredSelection selection = (IStructuredSelection) completionsViewer.getSelection();
		if(!selection.iterator().hasNext())
			return;
		Transformation transformation = (Transformation) selection.iterator().next();
		EditCompletionDialog dialog = new EditCompletionDialog(getShell(),transformation);
		if(dialog.open() == EditCompletionDialog.OK) {
			completionsViewer.refresh();
		}
		
	}

	@SuppressWarnings("unchecked")
	protected void removeCompletion() {
		IStructuredSelection selection = (IStructuredSelection) completionsViewer.getSelection();
		completionsViewer.getControl().setRedraw(false);
		for(Iterator i = selection.iterator();i.hasNext();) {
			Transformation transformation = (Transformation) i.next();
			transformations.remove(transformation);
			completionsViewer.refresh();
		}
		setTabModified();
		completionsViewer.getControl().setRedraw(true);
		
	}

	protected void newCompletion() {
		NewCompletionWizard wizard = new NewCompletionWizard();
		WizardDialog dialog = new WizardDialog(getShell(), wizard);
		dialog.setPageSize(500,500);
		if(dialog.open() == WizardDialog.OK) {
			transformations.add(wizard.getTransformation());
			completionsViewer.refresh();
			setTabModified();
		}
		/*Transformation newTransformation = new Transformation();
		EditCompletionDialog dialog = new EditCompletionDialog(getShell(),newTransformation);
		if(dialog.open() == EditCompletionDialog.OK) {
			transformations.add(newTransformation);
			completionsViewer.refresh();
			setTabModified();
		}*/
		
		
	}

	private TableViewer createTable(Composite mainComponent, String[] strings) {
		final TableViewer tableviewer;
		
		tableviewer = CheckboxTableViewer.newCheckList(mainComponent, SWT.BORDER | SWT.SINGLE | SWT.FULL_SELECTION);
		createColumns(tableviewer);
		tableviewer.setContentProvider(new ArrayContentProvider());
		tableviewer.setLabelProvider(new TransformationsLabelProvider());
		transformations = new ArrayList<Transformation>();
		
		tableviewer.setInput(transformations);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.verticalSpan = 6;
		tableviewer.getTable().setLayoutData(gd);
		
		((CheckboxTableViewer)tableviewer).addCheckStateListener(new ICheckStateListener() {
			public void checkStateChanged(CheckStateChangedEvent event) {
				Transformation entry = (Transformation)event.getElement();
				entry.enabled = event.getChecked();
				updateLaunchConfigurationDialog();
			}
		});
		
		return tableviewer;
	}
	
	private void createColumns(TableViewer viewer) {
		String[] viewerColumnTitles = {"Active", "Name","Category", "Element"};
		int[] columnWidths = { 50, 200, 200, 200};
		
		for(int i = 0; i < viewerColumnTitles.length; ++i) {
			TableViewerColumn column = new TableViewerColumn(viewer, SWT.NONE);
			column.getColumn().setText(viewerColumnTitles[i]);
			column.getColumn().setWidth(columnWidths[i]);
		}
		
		Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getName()
	 */
	@Override
	public String getName() {
		return "CIP Completions";
	}
	
	@Override
	public Image getImage() {
		return RunConfigImages.getCompletionTabImage();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#initializeFrom(org.eclipse.debug.core.ILaunchConfiguration)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void initializeFrom(ILaunchConfiguration configuration) {
		ArrayList<String> config = new ArrayList<String>();
		transformations.clear();
		try {
			config = (ArrayList<String>) configuration.getAttribute(ConstantsContainer.COMPLETION_CONFIG,new ArrayList<String>());
		} catch (CoreException e) {
			e.printStackTrace();
		}
		for (String transformationconfig : config) {
			Transformation transformation = new Transformation();
			transformation.readFromDataString(transformationconfig);
			transformations.add(transformation);
		}
		completionsViewer.refresh();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#performApply(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
	 */
	@Override
	public void performApply(ILaunchConfigurationWorkingCopy configuration) {
		ArrayList<String> config = new ArrayList<String>();
		for (Transformation transformation : transformations) {
			config.add(transformation.toDataString());
		}
		configuration.setAttribute(ConstantsContainer.COMPLETION_CONFIG, config);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#setDefaults(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
	 */
	@Override
	public void setDefaults(ILaunchConfigurationWorkingCopy configuration) {
		// TODO Auto-generated method stub

	}
	
	private class TransformationsLabelProvider extends LabelProvider implements ITableLabelProvider {

		@Override
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex) {
			Transformation transformation = (Transformation) element;
			switch (columnIndex) {
			case 1:
				if(transformation.completionPath != null)
					return transformation.completionPath;
				else
					return transformation.configPath;
			default:
				break;
			}
			return null;
		}
		
	}
	
	private class TransformationCellEditor extends DialogCellEditor {

		/**
		 * @param parent
		 */
		public TransformationCellEditor(Composite parent) {
			super(parent);
		}

		@Override
		protected Object openDialogBox(Control cellEditorWindow) {
			String str = CipCompletionTab.openModelDialog(cellEditorWindow.getShell());
			return str;
		}
		
	}
	
	public static String openModelDialog(Shell shell){
		String ret = null;
		ElementTreeSelectionDialog dialog = new ElementTreeSelectionDialog(shell, new WorkbenchLabelProvider(), new WorkbenchContentProvider());
        dialog.setTitle("Select a Resource:"); 
        dialog.setMessage("Select Models:"); 
        dialog.setInput(ResourcesPlugin.getWorkspace().getRoot()); 
        //dialog.setComparator(new ResourceComparator(ResourceComparator.NAME));
        if (dialog.open() == IDialogConstants.OK_ID) {
            Object[] resources = (Object[]) dialog.getResult();
            StringBuffer arg = new StringBuffer();
            IResource resource = null;
            for(int i = 0; i < (resources.length - 1); ++i){
            	resource = (IResource) resources[i];
            	arg.append(URI.createPlatformResourceURI(resource.getFullPath().toString(),true));
            	arg.append(" ");
            }
            resource = (IResource)resources[resources.length - 1];
            arg.append(URI.createPlatformResourceURI(resource.getFullPath().toString(),true));
            ret = arg.toString();
        }
        return ret;
	}

}
