/**
 * 
 */
package de.uka.ipd.sdq.cip.workflow.jobs;

import java.util.Iterator;

import org.eclipse.core.runtime.IProgressMonitor;

import de.uka.ipd.sdq.cip.configuration.CompletionConfiguration;
import de.uka.ipd.sdq.cip.configuration.CompletionConfigurationProvider;
import de.uka.ipd.sdq.cip.configuration.Transformation;
import de.uka.ipd.sdq.cip.configuration.Transformation.TransformationType;
import de.uka.ipd.sdq.cip.workflow.jobs.builder.HotCompletionBuilder;
import de.uka.ipd.sdq.cip.workflow.jobs.builder.PlainCompletionBuilder;
import de.uka.ipd.sdq.workflow.IBlackboardInteractingJob;
import de.uka.ipd.sdq.workflow.OrderPreservingBlackboardCompositeJob;
import de.uka.ipd.sdq.workflow.exceptions.JobFailedException;
import de.uka.ipd.sdq.workflow.exceptions.UserCanceledException;
import de.uka.ipd.sdq.workflow.mdsd.blackboard.MDSDBlackboard;

/**
 * @author thomasS
 * 
 */
public class CompletionJob extends
		OrderPreservingBlackboardCompositeJob<MDSDBlackboard> implements
		IBlackboardInteractingJob<MDSDBlackboard> {

	private CompletionConfiguration configuration;

	public CompletionJob(CompletionConfiguration configuration) {
		super();
		this.configuration = configuration;
	}

	public CompletionJob(CompletionConfigurationProvider configurationProvider) {
		super();
		this.configuration = configurationProvider.getCompletionConfiguration();
	}

	@Override
	public void execute(IProgressMonitor monitor) throws JobFailedException,
			UserCanceledException {

		Iterator<Transformation> iterator = configuration.getTransformations()
				.iterator();

		if (!iterator.hasNext())
			return;

		addJob(new CreateProjectJob(configuration));

		configuration.setLastPartition(this.myBlackboard
				.getPartition(configuration.getInputPartitionName()));

		while (iterator.hasNext()) {
			Transformation transformation = iterator.next();

			if (transformation.type == TransformationType.HOT) {
				HotCompletionBuilder hotBuilder = new HotCompletionBuilder(
						configuration, transformation);
				addAll(hotBuilder.buildJobs());
			} else {
				PlainCompletionBuilder hotBuilder = new PlainCompletionBuilder(
						configuration, transformation);
				addAll(hotBuilder.buildJobs());
			}
		}

		if (configuration.getAfterJobs() != null)
			addAll(configuration.getAfterJobs());

		super.execute(monitor);
	}

	// configuration.setLastPartition(configuration.getModelPartition());

	/*
	 * Iterator<CompletionEntry> iterator =
	 * configuration.getCompletionEntrys().iterator(); while(iterator.hasNext())
	 * { CompletionEntry completionEntry = iterator.next();
	 * 
	 * if(!completionEntry.isActive()) continue;
	 * 
	 * this.addJob(new CompletionTransformationJob(completionEntry,
	 * configuration)); }
	 */

	/*
	 * if(configuration.getActiveCompletionCount() > 0) this.addJob(new
	 * SplitJob(configuration));
	 * 
	 * 
	 * }
	 */

	/*
	 * @Override public void execute(IProgressMonitor monitor) throws
	 * JobFailedException, UserCanceledException { // get project from first
	 * model file IProject project =
	 * ResourcesPlugin.getWorkspace().getRoot().getProject
	 * (ConstantsContainer.outputPartition); //URI uri =
	 * this.myBlackboard.getPartition
	 * (configuration.getModelPartition()).getResourceSet
	 * ().getResources().get(1).getURI(); //IProject project =
	 * ResourcesPlugin.getWorkspace().getRoot().getFolder(new
	 * Path(uri.toFileString())).getProject();
	 * 
	 * try { if(project.exists()) project.delete(true, true, monitor);
	 * project.create(monitor); project.open(monitor); } catch (CoreException e)
	 * { // TODO Auto-generated catch block e.printStackTrace(); }
	 * 
	 * configuration.setProject(project);
	 * 
	 * this.myBlackboard.getPartition(configuration.getModelPartition()).
	 * initialiseResourceSetEPackages(ConstantsContainer.PCM_EPACKAGES);
	 * super.execute(monitor); }
	 */

}