package de.uka.ipd.sdq.cip.workflow.jobs.builder;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import de.uka.ipd.sdq.cip.configuration.CompletionConfiguration;
import de.uka.ipd.sdq.cip.configuration.Transformation;
import de.uka.ipd.sdq.workflow.IJob;
import de.uka.ipd.sdq.workflow.mdsd.blackboard.ResourceSetPartition;
import de.uka.ipd.sdq.workflow.mdsd.emf.qvtr.QVTRScript;
import de.uka.ipd.sdq.workflow.mdsd.emf.qvtr.QVTRScriptInfo;

public abstract class CompletionBuilder {
	
	protected CompletionConfiguration configuration;
	protected Transformation transformation;
	protected ArrayList<IJob> jobs ;
	
	public CompletionBuilder(CompletionConfiguration configuration, Transformation transformation) {
		this.configuration = configuration;
		this.transformation = transformation;
		jobs = new ArrayList<IJob>();
	}
	
	abstract public Collection<IJob> buildJobs();
	
	/**
	 * returns a new project to be used for the simulation
	 * 
	 * @return a handle to the project to be used for the simulation
	 */
	protected static IProject getProject(String projectId) {
		return ResourcesPlugin.getWorkspace().getRoot().getProject(
				projectId);
	}
	
	protected static ResourceSetPartition createAndLoadPartition(URI[] models) {
		ResourceSetPartition resourceSetPartition = new ResourceSetPartition();
		for(URI uri : models) {
			resourceSetPartition.loadModel(uri);
		}
		return resourceSetPartition;
	}
	
	protected static QVTRScript createQVTScript(String script, Collection<Object> metaModels, String transformation, String direction) {
		QVTRScript qvtrScript = new QVTRScript();
		qvtrScript.setQVTFile(script);
		qvtrScript.setMetaModels(metaModels);
		
		QVTRScriptInfo scriptinfo = new QVTRScriptInfo(qvtrScript);
		if(transformation == null)
			transformation = scriptinfo.getTransformations()[0];
		if(direction == null)
			direction = scriptinfo.getDirections(transformation)[1];
		
		qvtrScript.setTransformationName(transformation);
		qvtrScript.setTransformationDirection(direction);
		
		return qvtrScript;
	}
	
	protected Collection<Object> getEPackages(String metamodel) {
		ArrayList<Object> list = new ArrayList<Object>();
		ResourceSet rset = new ResourceSetImpl();
		Resource resource = rset.getResource(URI.createURI(metamodel),true);
		TreeIterator<EObject> iterator = resource.getAllContents();
		for(EObject object = null; iterator.hasNext(); object = iterator.next() ) {
			if(object instanceof EPackage)
				list.add(object);
		}
		
		return list;
	}
}
