package de.uka.ipd.sdq.dsexplore.analysis;

import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.opt4j.core.Objective;

import de.uka.ipd.sdq.dsexplore.helper.ExtensionHelper;
import de.uka.ipd.sdq.dsexplore.launch.DSEConstantsContainer;
import de.uka.ipd.sdq.dsexplore.launch.DSELaunch;
import de.uka.ipd.sdq.workflow.exceptions.JobFailedException;
import de.uka.ipd.sdq.workflow.exceptions.UserCanceledException;
import de.uka.ipd.sdq.workflow.mdsd.blackboard.MDSDBlackboard;

/** Singleton */
public class AnalysisProxy implements IAnalysis {

	private ILaunch launch;
	private ILaunchConfiguration configuration;
	
	private String qualityAttribute;
	
	IAnalysis ana = null;
	private IProgressMonitor monitor;
	
	//TODO: Quickfix to reset the Loggers. Refactor!
	private DSELaunch dseLaunch;
	private MDSDBlackboard blackboard;
	
	/** Logger for log4j. */
	private static Logger logger = 
		Logger.getLogger("de.uka.ipd.sdq.dsexplore.analysis.AnalysisProxy");

	public AnalysisProxy(MDSDBlackboard blackboard, ILaunchConfiguration configuration, ILaunch launch, IProgressMonitor monitor, String qualityAttribute, DSELaunch dseLaunch) {
		this.initialise(blackboard, configuration, launch, monitor);
		this.qualityAttribute = qualityAttribute;
		this.dseLaunch = dseLaunch;
	}

	@Override
	public void analyse() throws CoreException, UserCanceledException, AnalysisFailedException, JobFailedException {
		IAnalysis ana = getDecoratedAnalysis();
		
		logger.debug("Starting analysis of "+qualityAttribute);
		ana.analyse();
		logger.debug("Finished analysis of "+qualityAttribute);
		dseLaunch.resetLoggers();

	}

	private IAnalysis getDecoratedAnalysis() throws CoreException {
		if (ana == null){
//			ana = (IAnalysis)ExtensionHelper.loadExtension("de.uka.ipd.sdq.dsexplore.analysis").iterator().next();
			
			// obtain the extension of the choosen analysis method 
			String methodName = configuration.getAttribute(DSEConstantsContainer.getAnalysisMethod(this.qualityAttribute), "");
			IExtension[] extensions = ExtensionHelper.loadAnalysisExtensions();
			for (IExtension ext : extensions) {
				IConfigurationElement[] elements = ext.getConfigurationElements();
				for (IConfigurationElement element : elements) {
					if (element.getName().equals("analysis")) { 
						// if extension fits to analysis method
						if (element.getAttribute("name").equals(methodName)) { 
							// obtain an analysis method instance  
							ana = (IAnalysis)ExtensionHelper.loadExecutableAttribute(element, "delegate");
							ana.initialise(blackboard, configuration, launch, monitor);
							return ana;
						}
					}
				}
			}
			throw new CoreException(new Status(Status.ERROR, "de.uka.ipd.sdq.dsexplore", "Cannot load analysis extension for "+methodName));
		} else 
			return ana; 
	}

	@Override
	public void initialise(MDSDBlackboard blackboard, ILaunchConfiguration configuration, ILaunch launch,IProgressMonitor monitor) {
		this.launch = launch;
		this.configuration = configuration;
		this.monitor = monitor;
		this.blackboard = blackboard;
		
	}


	@Override
	public String getQualityAttribute() throws CoreException {
		return getDecoratedAnalysis().getQualityAttribute();
}

	@Override
	public boolean hasStatisticResults() throws CoreException {
		return this.getDecoratedAnalysis().hasStatisticResults();
	}

	@Override
	public List<Objective> getObjectives() throws CoreException {
		return getDecoratedAnalysis().getObjectives();
	}

	@Override
	public IAnalysisResult retrieveLastResultsForObjective(Objective objective)
			throws CoreException, AnalysisFailedException {
		return getDecoratedAnalysis().retrieveLastResultsForObjective(objective);
	}

}
