package de.uka.ipd.sdq.dsexplore.analysis.lqn;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

import LqnCore.LqnModelType;
import LqnCore.OutputResultType;
import LqnCore.ProcessorType;
import LqnCore.TaskType;
import de.uka.ipd.sdq.dsexplore.analysis.AnalysisFailedException;
import de.uka.ipd.sdq.dsexplore.analysis.IAnalysisResult;
import de.uka.ipd.sdq.dsexplore.analysis.UsageScenarioBasedObjective;
import de.uka.ipd.sdq.pcm.resourceenvironment.ProcessingResourceSpecification;
import de.uka.ipd.sdq.pcm.resourceenvironment.ResourceContainer;
import de.uka.ipd.sdq.pcm.resultdecorator.ResultDecoratorRepository;
import de.uka.ipd.sdq.pcm.resultdecorator.ResultdecoratorFactory;
import de.uka.ipd.sdq.pcm.resultdecorator.resourceenvironmentdecorator.ProcessingResourceSpecificationResult;
import de.uka.ipd.sdq.pcm.resultdecorator.resourceenvironmentdecorator.ResourceenvironmentdecoratorFactory;
import de.uka.ipd.sdq.pcmsolver.models.PCMInstance;
import de.uka.ipd.sdq.pcmsolver.transformations.pcm2lqn.Pcm2LqnHelper;

public abstract class LQNResult implements ILQNResult{
	
	protected static Logger logger = Logger
	.getLogger("de.uka.ipd.sdq.dsexplore");
	
	protected double responseTime;
	protected double squaredCoeffVariance = 1;

	private ResultDecoratorRepository results;

	public LQNResult(PCMInstance pcm, LqnModelType model,UsageScenarioBasedObjective objective) throws AnalysisFailedException {
		
		try{
			this.responseTime = retrieveResponseTimeForUsageScenario(pcm, model,objective);
			this.results =  retrieveResourceUtilisation(pcm, model);
		} catch (ParseException ex) {
			throw new AnalysisFailedException("Failed to parse string value.",
					ex);
		}
	}
	
	private ResultDecoratorRepository retrieveResourceUtilisation(
			PCMInstance pcm, LqnModelType model) throws ParseException {
		ResultDecoratorRepository repo = ResultdecoratorFactory.eINSTANCE.createResultDecoratorRepository();
		
		List<ProcessorType> processors = model.getProcessor();
		List<ResourceContainer> containers = pcm.getResourceEnvironment().getResourceContainer_ResourceEnvironment();
		
		//create a new list from that we can remove things without modifying the model
		List<ProcessorType> modifiableProcessorList = new ArrayList<ProcessorType>(processors.size());
		modifiableProcessorList.addAll(processors);
		
		for (ResourceContainer container : containers) {
			for (ProcessingResourceSpecification pcmResource : container.getActiveResourceSpecifications_ResourceContainer()) {
				String processorID = Pcm2LqnHelper.getIdForProcResource(container, pcmResource.getActiveResourceType_ActiveResourceSpecification())+"_Processor";
				
				//use iterator to be allowed to modify the list while searching
				Iterator<ProcessorType> lqnProcessorIterator = modifiableProcessorList.iterator();
				while (lqnProcessorIterator.hasNext()){
					ProcessorType lqnProc = lqnProcessorIterator.next();
					if (lqnProc.getName().equals(processorID)){
						ProcessingResourceSpecificationResult result = ResourceenvironmentdecoratorFactory.eINSTANCE.createProcessingResourceSpecificationResult();
						List<OutputResultType> lqnResultProc = lqnProc.getResultProcessor();
						if (lqnResultProc.size() > 0){
							String utilString = (String)lqnResultProc.get(0).getUtilization();
							result.setNormalisedResourceUtilisation(LQNUtils
									.convertStringToDouble(utilString));
							logger.debug("Resource "+processorID+" has utilisation "+utilString);
						}
						//TODO: implement these if possible.
						result.setNormalisedDemandedTime(Double.NaN);
						result.setNormalisedWaitTime(Double.NaN);
						result.setProcessingresourcespecification(pcmResource);
						result.setEntityName("Utilisation of "+processorID);
						repo.getUtilisationresult().add(result);
						
						//we can remove this element now and do not have to iterate over this one again in the next loop iterations. 
						lqnProcessorIterator.remove();
					}
				}
			}
		}
		return repo;
	}

	private double retrieveResponseTimeForUsageScenario(PCMInstance pcm, LqnModelType model, UsageScenarioBasedObjective objective) throws ParseException {
	
		// Retrieve the usage scenario's name used within the result
		// TODO: Can we really assume there is only one usage scenario?
		String scenarioName = Pcm2LqnHelper.getIdForUsageScenario(objective.getUsageScenario()); 
		String processorNameRegex = scenarioName + "_*_Processor"; // see class LqnBuilder#getProcessorTypeFromModel(String)
		
		// Obtain processor, representing the usage scenario's overall resource demand
		ProcessorType processor = null;
		Iterator<ProcessorType> itProcessors = model.getProcessor().iterator();
		while (itProcessors.hasNext()) {
			ProcessorType proc = itProcessors.next();
			if (proc.getName().matches(processorNameRegex)) {
				processor = proc;
				break;
			}
		}
		
		// TODO: Can we really assume there is only one task?
		TaskType task = processor.getTask().get(0);
		OutputResultType outputResult = task.getResultTask().get(0);
		
		double responseTime = 0.0;
		
		responseTime = LQNUtils.getReponseTimeOfSubActivities(task);

		if (outputResult.getSquaredCoeffVariation() != null){
			this.squaredCoeffVariance = LQNUtils
			.convertStringToDouble((String) outputResult
					.getSquaredCoeffVariation());
		} else 
			this.squaredCoeffVariance = 1;
		
		return responseTime;

	}


	/* (non-Javadoc)
	 * @see de.uka.ipd.sdq.dsexplore.analysis.lqn.ILQNResult#getMedianValue()
	 */
	public double getMedianValue() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see de.uka.ipd.sdq.dsexplore.analysis.lqn.ILQNResult#getMeanValue()
	 */
	public double getMeanValue() {
		return responseTime;
	}
	
	/* (non-Javadoc)
	 * @see de.uka.ipd.sdq.dsexplore.analysis.lqn.ILQNResult#getSquaredCoefficientOfVariance()
	 */
	public double getSquaredCoefficientOfVariance(){
		return this.squaredCoeffVariance;
	}
	
	/* (non-Javadoc)
	 * @see de.uka.ipd.sdq.dsexplore.analysis.lqn.ILQNResult#getCoefficientOfVariance()
	 */
	public double getCoefficientOfVariance(){
		return  Math.sqrt(this.squaredCoeffVariance);
	}
	
	/* (non-Javadoc)
	 * @see de.uka.ipd.sdq.dsexplore.analysis.lqn.ILQNResult#getVariance()
	 */
	public double getVariance(){
		double var = this.getSquaredCoefficientOfVariance() * this.getMeanValue() * this.getMeanValue(); 
		return var;
	}
	
	/* (non-Javadoc)
	 * @see de.uka.ipd.sdq.dsexplore.analysis.lqn.ILQNResult#getStandardDeviation()
	 */
	public double getStandardDeviation(){
		double std = Math.sqrt(this.getVariance()); 
		return std;
	}
	
	
	/* (non-Javadoc)
	 * @see de.uka.ipd.sdq.dsexplore.analysis.lqn.ILQNResult#getResults()
	 */
	public ResultDecoratorRepository getResults() {
		return results;
	}
	
	/* (non-Javadoc)
	 * @see de.uka.ipd.sdq.dsexplore.analysis.lqn.ILQNResult#compareTo(de.uka.ipd.sdq.dsexplore.analysis.IAnalysisResult)
	 */
	public int compareTo(IAnalysisResult other) {
		int res = 0;
		res = Double.valueOf(getMeanValue())
				.compareTo(other.getMeanValue());
		
		return res;
	}

}
