package de.uka.ipd.sdq.dsexplore.helper;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.common.util.EList;
import org.opt4j.core.Objective;

import de.uka.ipd.sdq.dsexplore.opt4j.representation.DSEIndividual;
import de.uka.ipd.sdq.dsexplore.opt4j.representation.DSEObjectives;
import de.uka.ipd.sdq.pcm.resultdecorator.ResultDecoratorRepository;
import de.uka.ipd.sdq.pcm.resultdecorator.resourceenvironmentdecorator.UtilisationResult;

public class UtilisationHelper {
	
	/**
	 * Gets utilisation result of an individual
	 * @param individual
	 * @return
	 */
	private static EList<UtilisationResult> getUtilisationResult(DSEIndividual individual) {
		DSEObjectives objectives = (DSEObjectives) individual.getObjectives();
		Objective resonseTimeObjective = null;
		for (Objective o : objectives.getKeys()) {
			if (o.getName().equals("mean response time")) {
				resonseTimeObjective = o;
			}
		}
		ResultDecoratorRepository resultDecoratorRepository = objectives.getResultDecoratorFor(resonseTimeObjective);
		
		if (resultDecoratorRepository != null) {
			return resultDecoratorRepository.getUtilisationresult();
		} else {
			return null;
		}
	}
	
	/**
	 * Returns maximum value of a Collection
	 * @param c Collection with elements of type Double
	 * @return Maximum element of c or Double.NEGATIVE_INFINITY of c is empty
	 */
	private static double getMaxElement(Collection<Double> c) {
		Double max = Double.NEGATIVE_INFINITY;
		for (Double d : c) {
			if (d > max) {
				max = d;
			}
		}
		return max;
	}
	
	/**
	 * Returns minimum value of a Collection
	 * @param c Collection with elements of type Double
	 * @return Minimum element of c or Double.POSITIVE_INFINITY of c is empty
	 */
	private static double getMinElement(Collection<Double> c) {
		Double min = Double.POSITIVE_INFINITY;
		for (Double d : c) {
			if (d < min) {
				min = d;
			}
		}
		return min;
	}
	
	public static double getMaxUtilisation(DSEIndividual individual){ 
		EList<UtilisationResult> utilisationResults = getUtilisationResult(individual);
		ArrayList<Double> results = new ArrayList<Double>();
		for (UtilisationResult utilisationResult : utilisationResults) {
			utilisationResult.getEntityName();
			results.add(utilisationResult.getNormalisedResourceUtilisation());
		}
		return getMaxElement(results);
	}
	
	public static double getMinUtilisation(DSEIndividual individual){ 
		EList<UtilisationResult> utilisationResults = getUtilisationResult(individual);
		ArrayList<Double> results = new ArrayList<Double>();
		for (UtilisationResult utilisationResult : utilisationResults) {
			results.add(utilisationResult.getNormalisedResourceUtilisation());
		}
		return getMinElement(results);
	}
	
	public static UtilisationResult getMaxUtilisationResult(DSEIndividual individual) {
		EList<UtilisationResult> utilisationResults = getUtilisationResult(individual);
		UtilisationResult maxUtilisationResult = null;
		if (utilisationResults != null) {
			for (UtilisationResult currentUtilisationResult : utilisationResults) {
				if (maxUtilisationResult == null) {
					maxUtilisationResult = currentUtilisationResult;
				}
				if (maxUtilisationResult.getNormalisedResourceUtilisation() < currentUtilisationResult.getNormalisedResourceUtilisation()) {
					maxUtilisationResult = currentUtilisationResult;
				}
			}
			return maxUtilisationResult;
		} else {
			return null;
		}
	}
	
	public static UtilisationResult getMinUtilisationResult(DSEIndividual individual) {
		EList<UtilisationResult> utilisationResults = getUtilisationResult(individual);
		UtilisationResult minUtilisationResult = null;
		if (utilisationResults != null) {
			for (UtilisationResult currentUtilisationResult : utilisationResults) {
				if (minUtilisationResult == null) {
					minUtilisationResult = currentUtilisationResult;
				}
				if (minUtilisationResult.getNormalisedResourceUtilisation() > currentUtilisationResult.getNormalisedResourceUtilisation()) {
					minUtilisationResult = currentUtilisationResult;
				}
			}
		return minUtilisationResult;
		} else {
			return null;
		}
	}
	
	/**
	 * Finds 2nd lowest UtilisationResult
	 * @param individual
	 * @return 2nd lowest UtilisationResult
	 */
	public static UtilisationResult get2ndMinUtilisationResult(DSEIndividual individual) {
		EList<UtilisationResult> utilisationResults = getUtilisationResult(individual);
		UtilisationResult result = null;
		UtilisationResult minUtilisationResult = getMinUtilisationResult(individual);
		if (utilisationResults != null) {
			for (UtilisationResult currentUtilisationResult : utilisationResults) {
				if (result == null && currentUtilisationResult != minUtilisationResult) {
					result = currentUtilisationResult;
				}
				if (result != null
						&& result.getNormalisedResourceUtilisation() > currentUtilisationResult.getNormalisedResourceUtilisation()
						&& currentUtilisationResult != minUtilisationResult) {
					result = currentUtilisationResult;
				}
			}
			return result;
		} else {
			return null;
		}
	}
	
}






