package de.uka.ipd.sdq.dsexplore.opt4j.optimizer.heuristic.operators;

import java.util.ArrayList;
import java.util.Collection;

import org.opt4j.core.problem.Genotype;
import org.opt4j.operator.copy.Copy;

import de.uka.ipd.sdq.dsexplore.launch.DSEWorkflowConfiguration;
import de.uka.ipd.sdq.dsexplore.opt4j.optimizer.heuristic.operators.impl.ProcessingRateImpl;
import de.uka.ipd.sdq.dsexplore.opt4j.optimizer.heuristic.operators.impl.ReallocationImpl;
import de.uka.ipd.sdq.dsexplore.opt4j.optimizer.heuristic.operators.impl.ServerConsolidationImpl;
import de.uka.ipd.sdq.dsexplore.opt4j.optimizer.heuristic.operators.impl.ServerExpansionImpl;
import de.uka.ipd.sdq.dsexplore.opt4j.representation.DSEIndividualBuilder;
import de.uka.ipd.sdq.dsexplore.opt4j.start.Opt4JStarter;

public class HeuristicFactory {
	
	/**
	 * Returns a collection of all actived heuristics.
	 */
	public static Collection<IHeuristic> getActivatedInstances(Copy<Genotype> copy, DSEIndividualBuilder individualBuilder, DSEWorkflowConfiguration configuration) {
		Collection<IHeuristic> activatedHeuristics = new ArrayList<IHeuristic>();
		if (configuration.getUseReallocation()) {
			activatedHeuristics.add(new ReallocationImpl(copy, individualBuilder, configuration));
		}
		if (configuration.getUseProcessingRate()) {
			activatedHeuristics.add(new ProcessingRateImpl(copy, individualBuilder, configuration));
		}
		if (configuration.getUseServerConsolidation()) {
			activatedHeuristics.add(new ServerConsolidationImpl(copy, individualBuilder, configuration));
		}
		if (configuration.getUseServerExpansion()) {
			activatedHeuristics.add(new ServerExpansionImpl(copy, individualBuilder, configuration));
		}
		return activatedHeuristics;
	}

}
