package de.uka.ipd.sdq.dsexplore.opt4j.start;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.opt4j.common.archive.BoundedArchive;
import org.opt4j.common.archive.CrowdingArchive;
import org.opt4j.common.archive.DefaultArchive;
import org.opt4j.config.Task;
import org.opt4j.config.Task.State;
import org.opt4j.core.Archive;
import org.opt4j.core.Individual;
import org.opt4j.core.IndividualBuilder;
import org.opt4j.core.IndividualCollection;
import org.opt4j.core.IndividualCollectionListener;
import org.opt4j.core.Population;
import org.opt4j.core.Value;
import org.opt4j.core.optimizer.Completer;
import org.opt4j.core.optimizer.Control;
import org.opt4j.core.optimizer.Optimizer;
import org.opt4j.core.problem.Evaluator;
import org.opt4j.optimizer.ea.EvolutionaryAlgorithmModule;
import org.opt4j.start.Opt4J;
import org.opt4j.start.Opt4JTask;

import com.google.inject.Module;

import de.uka.ipd.sdq.dsexplore.analysis.IAnalysis;
import de.uka.ipd.sdq.dsexplore.helper.ResultsWriter;
import de.uka.ipd.sdq.dsexplore.launch.DSEWorkflowConfiguration;
import de.uka.ipd.sdq.dsexplore.opt4j.archive.PopulationTracker;
import de.uka.ipd.sdq.dsexplore.opt4j.archive.PopulationTrackerModule;
import de.uka.ipd.sdq.dsexplore.opt4j.genotype.DesignDecisionGenotype;
import de.uka.ipd.sdq.dsexplore.opt4j.representation.DSECreator;
import de.uka.ipd.sdq.dsexplore.opt4j.representation.DSEDecoder;
import de.uka.ipd.sdq.dsexplore.opt4j.representation.DSEEvaluator;
import de.uka.ipd.sdq.dsexplore.opt4j.representation.DSEEvolutionaryAlgorithmModule;
import de.uka.ipd.sdq.dsexplore.opt4j.representation.DSEIndividual;
import de.uka.ipd.sdq.dsexplore.opt4j.representation.DSEIndividualBuilder;
import de.uka.ipd.sdq.dsexplore.opt4j.representation.DSEIndividualStateListener;
import de.uka.ipd.sdq.dsexplore.opt4j.representation.DSEModule;
import de.uka.ipd.sdq.dsexplore.opt4j.representation.DSEProblem;
import de.uka.ipd.sdq.pcm.cost.CostRepository;
import de.uka.ipd.sdq.pcm.designdecision.Choice;
import de.uka.ipd.sdq.pcmsolver.models.PCMInstance;

//import org.aopalliance.intercept.MethodInterceptor;

/**
 * Main class to start the whole Opt4J stuff. 
 * 
 * TODO: Make this a Singleton instead of static? Even better: Handle the global stuff properly. Similar to SimuComModel.
 * @author martens
 *
 */
public class Opt4JStarter {
	
	private static DSEProblem problem = null;
	private static DSECreator creator = null;
	
	private static Opt4JTask task = null;
	
	private static DSEWorkflowConfiguration myDSEConfig = null;

	/** Logger for log4j. */
	private static Logger logger = 
		Logger.getLogger("de.uka.ipd.sdq.dsexplore.opt4j.start.Opt4JStarter");

	private static List<Value<Double>> upperConstraints;

	private static List<ResultsWriter> writers = null;
	
	public static void init(List<IAnalysis> evaluators, List<Value<Double>> upperConstraints, DSEWorkflowConfiguration dseConfig, PCMInstance pcmInstance) throws CoreException{
		
		Opt4JStarter.myDSEConfig = dseConfig;
		
		Collection<Module> modules = new ArrayList<Module>();

		DSEModule dseModule = new DSEModule();
		modules.add(dseModule);

		addOptimisationModules(dseConfig.getMaxIterations(), dseConfig,
				modules);

		addPopulationModule(modules);
		
		Opt4JStarter.task = new Opt4JTask(false);
		task.init(modules);
		task.open();
		
		Opt4JStarter.setUpperConstraints(upperConstraints);
		Opt4JStarter.writers = new LinkedList<ResultsWriter>();
		
		Opt4JStarter.problem = new DSEProblem(dseConfig, pcmInstance);
		if (dseConfig.isNewProblem()){
			Opt4JStarter.problem.saveProblem();
		} 
		
		DSEEvaluator ev = task.getInstance(DSEEvaluator.class);
		ev.init(evaluators);
		
	}
		
	/**
	 * Only starts Opt4J, needs call to {@link Opt4JStarter#init(IAnalysis, IAnalysis, List, CostRepository, PCMInstance, boolean)} first.
	 * @param dseConfig
	 *@param monitor
	 * @param genotypes May be null
	 * @param allCandidates 
	 * @throws CoreException
	 */
	public static void runOpt4JWithPopulation(
			DSEWorkflowConfiguration dseConfig, 
			IProgressMonitor monitor,
			List<DesignDecisionGenotype> genotypes, 
			List<DesignDecisionGenotype> allCandidates) throws CoreException {
		
		if (Opt4JStarter.problem == null){
			throw new CoreException(new Status(Status.ERROR,
					"de.uka.ipd.sdq.dsexplore", 0, "Opt4JStarter has not been properly initialised. Contact developers.", null));
		}
		
		if (genotypes != null && genotypes.size() > 0){
			Opt4JStarter.problem.setInitialPopulation(genotypes);
		}
		
		DSEListener listener = new DSEListener(monitor,dseConfig);
		runTask(listener, dseConfig, allCandidates);

	}

	/**
	 * inits and starts Opt4J
	 * @param perfAnalysisTool
	 * @param relAnalysisTool
	 * @param pcmInstance
	 * @param upperConstraints
	 * @param monitor
	 * @param pcmInstance 
	 * @param newProblem
	 * @throws CoreException
	 */
	public static void startOpt4J(List<IAnalysis> evaluators, DSEWorkflowConfiguration dseConfig, List<Value<Double>> upperConstraints, IProgressMonitor monitor, PCMInstance pcmInstance)
			throws CoreException {
		
		runOpt4JWithPopulation(dseConfig, monitor, null, null);
		
	}

	/**
	 * 
	 * @param listener
	 * @param dseConfig
	 * @param allCandidates may be null
	 * @throws CoreException
	 */
	private static void runTask(DSEListener listener, DSEWorkflowConfiguration dseConfig, List<DesignDecisionGenotype> allCandidates)
			throws CoreException {
		
		try {
			Opt4JStarter.creator = task.getInstance(DSECreator.class);
			Optimizer opt = task.getInstance(Optimizer.class);
			opt.addOptimizerIterationListener(listener);
			IndividualBuilder indivBuilder = task.getInstance(IndividualBuilder.class);
			indivBuilder.addIndividualStateListener(new DSEIndividualStateListener(dseConfig));
			
			//add the previously defined all candidates to the custom PopulationTracker archive, 
			PopulationTracker populationTracker = getAllIndividuals();
			Completer completer = task.getInstance(Completer.class);
			if (allCandidates != null && allCandidates.size() > 0){
				if (indivBuilder instanceof DSEIndividualBuilder){
					logger.debug("Reading in predefined all candidates.");
					DSEIndividualBuilder dseBuilder = (DSEIndividualBuilder)indivBuilder;
					for (DesignDecisionGenotype designDecisionGenotype : allCandidates) {
						DSEIndividual individual = dseBuilder.build(designDecisionGenotype);
						completer.complete(individual);
						populationTracker.addIndividualsManually(individual);
					}
					logger.debug("Finished reading in predefined all candidates.");
					
				}
				
			}
			
			task.execute();
			
			
		} catch (CoreException e) {
			throw e;
		} catch (Exception e) {
			throw new CoreException(new Status(Status.ERROR,
					"de.uka.ipd.sdq.dsexplore", 0, e.getMessage(), e));
		} finally {
			try {

			
//			PopulationTracker allIndividuals = getAllIndividuals();
//			ResultsWriter.printOutIndividuals(allIndividuals.getIndividuals(), "All Individuals");
//			
//			ResultsWriter.printOutIndividuals(allIndividuals.getParetoOptimalIndividuals(), "Own Optimal Candidates");
			
			} catch (Exception e){
				logger.error("Optimisation failed, I could not save the results.");
				e.printStackTrace();
			}
			
				
			DSEEvaluator evaluator = task.getInstance(DSEEvaluator.class);
			List<Exception> exceptions = evaluator.getExceptionList();
			
			String config = dseConfig.getOriginalConfiguration().getMemento();
			ResultsWriter.writeStringToFile("config", config, listener.getIteration(), exceptions);
			
			if (exceptions.size() > 0){
				logger.warn("Errors occured during evaluation.");
				for (Exception exception : exceptions) {
					exception.printStackTrace();
				}
			}
			
		}
	}

	private static void addPopulationModule(Collection<Module> modules) {
		// ArchiveModule am = new ArchiveModule();
		// am.setType(ArchiveModule.Type.);
		PopulationTrackerModule p = new PopulationTrackerModule();
		
		modules.add(p);
		// modules.add(dtlz);
		// modules.add(gui);
	}
		
	private static void addOptimisationModules(int maxIterations,
			DSEWorkflowConfiguration config,
			Collection<Module> modules) {
		
		int individualsPerGeneration = config.getIndividualsPerGeneration();
		double crossoverRate = config.getCrossoverRate();
		
		EvolutionaryAlgorithmModule ea = new DSEEvolutionaryAlgorithmModule();
		ea.setGenerations(maxIterations);
		ea.setAlpha(individualsPerGeneration);
		ea.setLambda((int) Math.floor(individualsPerGeneration / 2.0 + 0.5));
		ea.setCrossoverRate(crossoverRate);

/*		SimulatedAnnealingModule sa = new SimulatedAnnealingModule();
		sa.setIterations(maxIterations);*/


		/*
		 * GUIModule gui = new GUIModule(); gui.setCloseOnStop(true);
		 */
		modules.add(ea);
	}


	public static void closeTask(){
		if (Opt4JStarter.task != null){
			Opt4JStarter.task.close(); 
			}
		}
		
	@Deprecated
	public static void startOpt4JWithGUI(){
		//String id = Opt4JPluginActivator.PLUGIN_ID;
		
			try {
			Opt4J.main(new String[0]);
			
			} catch (Exception e){
			// TODO Auto-generated catch block
			e.printStackTrace();
			}
		
			}

	public synchronized static void terminate (){
		if (task != null && !task.getState().equals(State.DONE)){
			Control control = task.getInstance(Control.class);
			control.doTerminate();	
			logger.warn("Terminating run");
		} else {
			logger.warn("Cannot terminate as no task is executing");
		}
		}

	/**
	 * Returns the instance of {@link Archive} from the Opt4J {@link Task},
	 *  which is a {@link DefaultArchive} inheriting from 
	 *  {@link CrowdingArchive} and {@link BoundedArchive}.  
	 * @return
	 */
	public static Archive getArchiveIndividuals(){
		return task.getInstance(Archive.class);
	}

	/**
	 * Returns the instance of {@link Population} from the Opt4J {@link Task}, 
	 * which is a plain {@link IndividualCollection}.  
	 * @return
	 */
	public static List<DSEIndividual> getPopulationIndividuals(){
		Population p = task.getInstance(Population.class);
		List<DSEIndividual> individuals = new ArrayList<DSEIndividual>(p.size());
		for (Individual individual : p) {
			if (individual instanceof DSEIndividual){
				individuals.add((DSEIndividual) individual);
			} else {
				logger.error("There was an illegal individual that is not instance of DSEIndividual. Ignoring it. "+individual.toString());
			}
		}
		return individuals; 
	}

	/**
	 * Returns the instance of {@link PopulationTracker} from the Opt4J {@link Task}, 
	 * which is an {@link IndividualCollectionListener} that listens on the 
	 * {@link Population} instance from the Opt4J {@link Task}.  
	 * @return
	 */
	public static PopulationTracker getAllIndividuals(){
		return task.getInstance(PopulationTracker.class);
		}
	
	public static void tearDown(){
		
		for (ResultsWriter writer : writers) {
			writer.close();
		}
		
		writers = null;
			
		problem = null;
		
		creator = null;
	
		task = null;

		setUpperConstraints(null);
		}

	public static void registerWriter(ResultsWriter writer) {
		writers.add(writer);
		
	}

	public static Choice createRandomGene(int index, DesignDecisionGenotype genotype) {
		if (Opt4JStarter.creator != null){
			return Opt4JStarter.creator.createRandomChoice(genotype.get(index).getDegreeOfFreedom());
		} else {
			logger.error("Could not create random value as DSECreator is not available.");
			throw new RuntimeException("Could not create random value as DSECreator is not available.");
		}
		
	}

	/**
	 * @return the problem
	 */
	public static DSEProblem getProblem() {
		return problem;
	}
	
	public static DSEEvaluator getDSEEvaluator() throws CoreException{
		Evaluator<?> e = task.getInstance(Evaluator.class);
		if (e != null && e instanceof DSEEvaluator){
			return (DSEEvaluator)e;
		} else {
			throw new CoreException(new Status(Status.ERROR,
					"de.uka.ipd.sdq.dsexplore", "Wrong initialisation of Evaluator: class DSEEvaluator expected, but found "+e == null ? "null" : e.getClass().getName()));
		}
	}
	
	public static DSEDecoder getDSEDecoder() throws CoreException{
		DSEDecoder e = task.getInstance(DSEDecoder.class);
		if (e != null){
			return e;
		} else {
			throw new CoreException(new Status(Status.ERROR,
					"de.uka.ipd.sdq.dsexplore", "Wrong initialisation of Decoder: class DSEDecoder expected, but found null"));
		}
	}

	/**
	 * @param upperConstraints the upperConstraints to set
	 */
	private static void setUpperConstraints(List<Value<Double>> upperConstraints) {
		Opt4JStarter.upperConstraints = upperConstraints;
	}

	/**
	 * @return the upperConstraints
	 */
	public static List<Value<Double>> getUpperConstraints() {
		return upperConstraints;
	}

	
	public static DSEWorkflowConfiguration getDSEWorkflowConfig() {
		return myDSEConfig;
	}
	
}
	
		
